import {Integer, isInteger} from './types'
import {ErrorWithContext} from '../error/ErrorWithContext'

export class Safe {
    protected thrower: (message: string, value: unknown) => never

    constructor(
        protected readonly value: unknown,
    ) {
        this.thrower = (message) => {
            throw new ErrorWithContext('Invalid value', {
                message,
                value,
            })
        }
    }

    onError(thrower: (message: string, value: unknown) => never): this {
        this.thrower = thrower
        return this
    }

    present(): this {
        if ( !this.value && this.value !== 0 && this.value !== false ) {
            return this
        }

        this.thrower('Missing value', this.value)
    }

    integer(): Integer {
        const value = parseInt(String(this.value), 10)
        if ( !isInteger(value) ) {
            this.thrower('Invalid integer', this.value)
        }

        return value
    }

    number(): number {
        const value = parseFloat(String(this.value))
        if ( isNaN(value) ) {
            this.thrower('Invalid number', this.value)
        }

        return value
    }

    string(): string {
        this.present()
        return String(this.value)
    }

    in<T>(allowed: T[]): T {
        if ( allowed.includes(this.value as any) ) {
            return this.value as T
        }

        this.thrower('Invalid value', this.value)
    }
}
